@@ -81,6 +81,7 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
     BOOL _playbackWillClose;
     BOOL _isTapSeeking;
     VLCMovieJumpState _previousJumpState;
+    VLCMediaPlayerState _previousPlayerStateWasPaused;
 
     UIPinchGestureRecognizer *_pinchRecognizer;
     VLCPanType _currentPanType;
@@ -431,7 +432,7 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
     VLCRendererDiscovererManager *manager = [VLCRendererDiscovererManager sharedInstance];
     manager.presentingViewController = self;
     manager.delegate = self;
-    if (_vpc.renderer || [[UIDevice currentDevice] VLCHasExternalDisplay]) {
+    if ([_vpc isPlayingOnExternalScreen]) {
          [self showOnDisplay:_playingExternalView.displayView];
     }
 }
@@ -778,6 +779,18 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
     [self setControlsHidden:!_controlsHidden animated:YES];
 }
 
+- (void)updateActivityIndicatorForState:(VLCMediaPlayerState)state
+{
+    if (state == VLCMediaPlayerStatePlaying || state == VLCMediaPlayerStatePaused) {
+        _previousPlayerStateWasPaused = state == VLCMediaPlayerStatePaused;
+    }
+    BOOL shouldAnimate = state == VLCMediaPlayerStateBuffering && !_previousPlayerStateWasPaused;
+    if (self.activityIndicator.isAnimating != shouldAnimate) {
+        self.activityIndicator.alpha = shouldAnimate ? 1.0 : 0.0;
+        shouldAnimate ? [self.activityIndicator startAnimating] : [self.activityIndicator stopAnimating];
+    }
+}
+
 - (void)_resetIdleTimer
 {
     if (!_idleTimer)
@@ -1002,6 +1015,9 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
 
 - (void)playbackPositionUpdated:(VLCPlaybackController *)controller
 {
+    // FIXME: hard coded state since the state in mediaPlayer is incorrectly still buffering
+    [self updateActivityIndicatorForState:VLCMediaPlayerStatePlaying];
+
     if (!_isScrubbing) {
         self.timeNavigationTitleView.positionSlider.value = [controller playbackPosition];
     }
@@ -1043,6 +1059,8 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
         currentMediaHasChapters:(BOOL)currentMediaHasChapters
           forPlaybackController:(VLCPlaybackController *)controller
 {
+    [self updateActivityIndicatorForState:currentState];
+
     if (currentState == VLCMediaPlayerStateBuffering)
         _mediaDuration = controller.mediaDuration;
 
