@@ -2,7 +2,7 @@
  * VLCPlaybackController.m
  * VLC for iOS
  *****************************************************************************
- * Copyright (c) 2013-2015 VideoLAN. All rights reserved.
+ * Copyright (c) 2013-2018 VideoLAN. All rights reserved.
  * $Id$
  *
  * Authors: Felix Paul Kühne <fkuehne # videolan.org>
@@ -63,12 +63,10 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
     BOOL _needsMetadataUpdate;
     BOOL _mediaWasJustStarted;
     BOOL _recheckForExistingThumbnail;
-    BOOL _headphonesWasPlugged;
+    BOOL _externalAudioPlaybackDevicePlugged;
 
     NSLock *_playbackSessionManagementLock;
 
-    VLCDialogProvider *_dialogProvider;
-
     NSMutableArray *_shuffleStack;
     void (^_playbackCompletion)(BOOL success);
 }
@@ -102,7 +100,7 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
     self = [super init];
     if (self) {
         // listen to audiosessions and appkit callback
-        _headphonesWasPlugged = [self areHeadphonesPlugged];
+        _externalAudioPlaybackDevicePlugged = [self isExternalAudioPlaybackDevicePlugged];
         NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
         [defaultCenter addObserver:self selector:@selector(audioSessionRouteChange:)
                               name:AVAudioSessionRouteChangeNotification object:nil];
@@ -119,7 +117,8 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
                               name:UIApplicationDidEnterBackgroundNotification object:nil];
 
         _metadata = [VLCMetaData new];
-        _dialogProvider = [[VLCDialogProvider alloc] initWithLibrary:[VLCLibrary sharedLibrary] customUI:NO];
+        _dialogProvider = [[VLCDialogProvider alloc] initWithLibrary:[VLCLibrary sharedLibrary] customUI:YES];
+        _dialogProvider.customRenderer = self;
 
         _playbackSessionManagementLock = [[NSLock alloc] init];
         _shuffleMode = NO;
@@ -193,10 +192,7 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
     _actualVideoOutputView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
     _actualVideoOutputView.autoresizesSubviews = YES;
 
-    if (_pathToExternalSubtitlesFile)
-        _listPlayer = [[VLCMediaListPlayer alloc] initWithOptions:@[[NSString stringWithFormat:@"--%@=%@", kVLCSettingSubtitlesFilePath, _pathToExternalSubtitlesFile]] andDrawable:_actualVideoOutputView];
-    else
-        _listPlayer = [[VLCMediaListPlayer alloc] initWithDrawable:_actualVideoOutputView];
+    _listPlayer = [[VLCMediaListPlayer alloc] initWithDrawable:_actualVideoOutputView];
 
     /* to enable debug logging for the playback library instance, switch the boolean below
      * note that the library instance used for playback may not necessarily match the instance
@@ -207,12 +203,8 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
     [_mediaPlayer setDelegate:self];
     if ([[defaults objectForKey:kVLCSettingPlaybackSpeedDefaultValue] floatValue] != 0)
         [_mediaPlayer setRate: [[defaults objectForKey:kVLCSettingPlaybackSpeedDefaultValue] floatValue]];
-    if ([[defaults objectForKey:kVLCSettingDeinterlace] intValue] != 0)
-        [_mediaPlayer setDeinterlaceFilter:@"blend"];
-    else
-        [_mediaPlayer setDeinterlaceFilter:nil];
-    if (_pathToExternalSubtitlesFile)
-        [_mediaPlayer addPlaybackSlave:[NSURL fileURLWithPath:_pathToExternalSubtitlesFile] type:VLCMediaPlaybackSlaveTypeSubtitle enforce:YES];
+    int deinterlace = [[defaults objectForKey:kVLCSettingDeinterlace] intValue];
+    [_mediaPlayer setDeinterlace:deinterlace withFilter:@"blend"];
 
     VLCMedia *media = [_mediaList mediaAtIndex:_itemInMediaListToBePlayedFirst];
     [media parseWithOptions:VLCMediaParseLocal];
@@ -259,6 +251,17 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
 
     [[self remoteControlService] subscribeToRemoteCommands];
 
+    if (_pathToExternalSubtitlesFile) {
+        /* this could be a path or an absolute string - let's see */
+        NSURL *subtitleURL = [NSURL URLWithString:_pathToExternalSubtitlesFile];
+        if (!subtitleURL) {
+            subtitleURL = [NSURL fileURLWithPath:_pathToExternalSubtitlesFile];
+        }
+        if (subtitleURL) {
+            [_mediaPlayer addPlaybackSlave:subtitleURL type:VLCMediaPlaybackSlaveTypeSubtitle enforce:YES];
+        }
+    }
+
     _playerIsSetup = YES;
 
     [[NSNotificationCenter defaultCenter] postNotificationName:VLCPlaybackControllerPlaybackDidStart object:self];
@@ -294,12 +297,6 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
     }
     if (!_sessionWillRestart) {
         _mediaList = nil;
-        if (_pathToExternalSubtitlesFile) {
-            NSFileManager *fileManager = [NSFileManager defaultManager];
-            if ([fileManager fileExistsAtPath:_pathToExternalSubtitlesFile])
-                [fileManager removeItemAtPath:_pathToExternalSubtitlesFile error:nil];
-            _pathToExternalSubtitlesFile = nil;
-        }
     }
     _playerIsSetup = NO;
     [_shuffleStack removeAllObjects];
@@ -1099,11 +1096,13 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
     }
 }
 
-- (BOOL)areHeadphonesPlugged
+- (BOOL)isExternalAudioPlaybackDevicePlugged
 {
+    /* check what output device is currently connected
+     * this code assumes that everything which is not a builtin speaker, must be external */
     NSArray *outputs = [[AVAudioSession sharedInstance] currentRoute].outputs;
-    NSString *portName = [[outputs firstObject] portName];
-    return [portName isEqualToString:@"Headphones"];
+    AVAudioSessionPortDescription *outputDescription = outputs.firstObject;
+    return ![outputDescription.portType isEqualToString:AVAudioSessionPortBuiltInSpeaker];
 }
 
 - (void)audioSessionRouteChange:(NSNotification *)notification
@@ -1114,16 +1113,17 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
     if (routeChangeReason == AVAudioSessionRouteChangeReasonRouteConfigurationChange)
         return;
 
-    BOOL headphonesPlugged = [self areHeadphonesPlugged];
+    BOOL externalAudioPlaybackDevicePlugged = [self isExternalAudioPlaybackDevicePlugged];
 
-    if (_headphonesWasPlugged && !headphonesPlugged && [_mediaPlayer isPlaying]) {
+    if (_externalAudioPlaybackDevicePlugged && !externalAudioPlaybackDevicePlugged && [_mediaPlayer isPlaying]) {
+        APLog(@"Pausing playback as previously plugged external audio playback device was removed");
         [_mediaPlayer pause];
 #if TARGET_OS_IOS
         [self _savePlaybackState];
 #endif
         [[NSNotificationCenter defaultCenter] postNotificationName:VLCPlaybackControllerPlaybackDidPause object:self];
     }
-    _headphonesWasPlugged = headphonesPlugged;
+    _externalAudioPlaybackDevicePlugged = externalAudioPlaybackDevicePlugged;
 }
 
 #pragma mark - Managing the media item
@@ -1228,6 +1228,11 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
     _sleepTimer = [NSTimer scheduledTimerWithTimeInterval:timeInterval target:self selector:@selector(stopPlayback) userInfo:nil repeats:NO];
 }
 
+- (BOOL)isPlayingOnExternalScreen
+{
+    return (_renderer || [[UIDevice currentDevice] VLCHasExternalDisplay]);
+}
+
 #pragma mark - background interaction
 
 - (void)applicationWillResignActive:(NSNotification *)aNotification
@@ -1235,8 +1240,8 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
 #if TARGET_OS_IOS
     [self _savePlaybackState];
 #endif
-
-    if (![[[NSUserDefaults standardUserDefaults] objectForKey:kVLCSettingContinueAudioInBackgroundKey] boolValue]) {
+    if (![self isPlayingOnExternalScreen]
+        && ![[[NSUserDefaults standardUserDefaults] objectForKey:kVLCSettingContinueAudioInBackgroundKey] boolValue]) {
         if ([_mediaPlayer isPlaying]) {
             [_mediaPlayer pause];
             _shouldResumePlaying = YES;
@@ -1345,5 +1350,4 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
     _renderer = renderer;
     [_mediaPlayer setRendererItem:_renderer];
 }
-
 @end
